<!--- 
	csw-map-google.cfc
		
	Version: [2018-01-02 01:28 pm]
---> 


<cfcomponent displayname="csw-map-google" output="false" hint="Google Map Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			Server.CommonSpot.udf.resources.loadResources("JQuery");
			Server.CommonSpot.udf.resources.loadResources("MaplaceJS");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");
			Server.CommonSpot.udf.resources.loadResources("cswImageStyles");
			Server.CommonSpot.udf.resources.loadResources("cswMapGoogle");

			// If a custom stylesheet has been registered, go ahead and load it.
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswImageStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswImageStyles_custom");

			if ( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswMapGoogle_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswMapGoogle_custom");
		</cfscript>
	</cffunction>


	
	<cffunction name="loadProperties" access="public" returntype="struct" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW Map";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();


			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;
			
			//
			// Get Metadata (Google Map metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];


			// Google Maps API key check
			if( len(widgetData.apikey) EQ 0 )
			{
				WriteOutput("<div><strong><em>Please open the Custom Properties for this Render Handler and add a Google Maps API key!</em></strong></div>");
				return props;
			}
			

			/**************************************/
			/**** MAP OPTIONS PROPERTIES TAB ******/
			/**************************************/

			// Width - optional
			props.width = getStandardField( widgetData=widgetData, field='width', defValue='csw100pct' );

			// Height - optional
			props.height = getStandardField( widgetData=widgetData, field='height', defValue='' );

			// Zoom - optional
			props.zoom = getStandardField( widgetData=widgetData, field='zoom', defValue='15' );

			// Alignment - optional
			props.alignment = getStandardField( widgetData=widgetData, field='Alignment', defValue='cswAlignCenter' );


			/**************************************/
			/**** GENERAL PROPERTIES TAB **********/
			/**************************************/

			// Element (STRUCT)
			props.overall = getFieldSettings(widgetData=widgetData, field='Overall', defBgColor='', defPadding='');

			// Background Image (STRUCT)
			props.backgroundImage = getStandardMetadataImageField( widgetData=widgetData, field='Background_Image', defValue='' );

			// Background Image Focal Point
			props.backgroundImageFocalPoint = getStandardField(widgetData=widgetData, field='Background_Image_Focal_Point', defValue='cswImgPos-centermiddle');


			/*******************************************/
			/**** API AUTHENTICATION PROPERTIES TAB ****/
			/*******************************************/

			// API Key
			props.apiKey = getStandardField(widgetData=widgetData, field='APIkey', defValue='');


			return props;
		</cfscript>
	</cffunction>


	
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;

			// Set the element ID
			var items = elementData.propertyValues;
			var elementID = request.element.id;

			var googleMapMarkersApiJS = "";
			var googleMapRenderJS = "";
			var markerSet = "";
			var styleAttr = "";
		</cfscript>

		<cfif !StructKeyExists(request,"cswMapMarkersGoogle") OR !StructKeyExists(request.cswMapMarkersGoogle,hash(props.APIkey))>
			<cfsavecontent variable="googleMapMarkersApiJS">
				<cfoutput>
					<script src="https://maps.google.com/maps/api/js?libraries=geometry&key=#props.APIkey#"></script>
				</cfoutput>
			</cfsavecontent>

			<cfscript>
				request.cswMapMarkersGoogle[hash(props.apiKey)] = 1;
				// Load custom javascript at the bottom of the page.
				Server.CommonSpot.udf.resources.addFooterHTML(googleMapMarkersApiJS, 'SECONDARY');
			</cfscript>
		</cfif>

		<cfscript>
			styleAttr = '';
			if( structKeyExists(props,'overall') )
			{
				if( len(props.overall.padding) )
					styleAttr = styleAttr & 'padding: #props.overall.padding#;';
				if( len(props.overall.bgColor) )
					styleAttr = styleAttr & 'background-color: #props.overall.bgColor#;';
			}

			if( StructKeyExists(props,'backgroundImage') AND len(props.backgroundImage) )
				styleAttr = styleAttr & "background-image: url('#props.backgroundImage#'); background-repeat: no-repeat; background-size: cover;";

			if( len(styleAttr) )
				styleAttr = 'style="#styleAttr#"';
		</cfscript>

		<cfoutput><div class="csw-Map-Google #props.alignment# #props.backgroundImageFocalPoint#" #styleAttr#></cfoutput>

		<cfset markerSet = getItemData(elementData)>

		<cfsavecontent variable="googleMapRenderJS">
			<cfoutput>
				<script type="text/javascript">
					jQuery(document).ready(function() {
						// marker set locations
						var markerLocs_#elementID# = #markerSet#;

						// Init Google Maps
						initGoogleWidgetMap(markerLocs_#elementID#, "##cswGoogleMap-#elementID#", #props.zoom#);
					});
				</script>
			</cfoutput>
		</cfsavecontent>

		<cfscript>
			// Load custom javascript at the bottom of the page.
			Server.CommonSpot.udf.resources.addFooterHTML(googleMapRenderJS, 'SECONDARY');
		</cfscript>

		<cfoutput>
			<div class="cswItems cswItem #props.width#">
				<div class="cswMap" id="cswGoogleMap-#elementID#" style="height:#props.height#;"></div>
			</div>
		</cfoutput>

		<cfoutput></div></cfoutput>

	</cffunction>
	
	

	<cffunction name="getItemData" access="public" output="Yes">
		<cfargument name="valuesStruct" type="struct" required="Yes">

		<cfscript>
			var items = arguments.valuesStruct.propertyValues;
			var totalItems = arraylen(items);
			var itm = 0;
			var currTitle = "";
			var currHTML = "";
			var animatedState = "";
			var markerData = StructNew();
			var markerArr = arrayNew(1);

			for ( itm=1; itm LTE totalItems; itm=itm+1 )
			{
				// Custom Element data coming from CS render mode filter so unescaping is needed before applying jsstringformat()
				currTitle = "";
				if ( StructKeyExists(items[itm].values,'title') AND LEN(TRIM(items[itm].values.title)) )
					currTitle = items[itm].values.title;
				currTitle = server.commonspot.udf.html.unescape(currTitle);
				currTitle = jsstringformat(currTitle);

				currHTML = "";
				if ( StructKeyExists(items[itm].values,'infoWindowHTML') AND LEN(TRIM(items[itm].values.infoWindowHTML)) )
					currHTML = items[itm].values.infoWindowHTML;
				currHTML = server.commonspot.udf.html.unescape(trim(currHTML));

				// Build the Marker Options
				animatedState = "";
				if ( StructKeyExists(items[itm].values,'animated') AND items[itm].values.animated EQ 1 )
					animatedState = " google.maps.Animation.DROP";

				// iconURL URL
				iconURL = items[itm].values.Icon;
				if (structKeyExists(items[itm].values,"Icon"))
				{
					iconURL = items[itm].values.Icon;
					if (isStruct(iconURL))  {
						if( StructKeyExists(iconURL,'ResolvedURL') )
							iconURL = iconURL.ResolvedURL.ServerRelative;
						else
							iconURL = trim(iconURL);
					}
					else
						iconURL = trim(iconURL);
				}

				markerData = StructNew();
				// IMPORTANT:  Use bracket notation to preserve case when converting to JSON
				markerData["lat"] = items[itm].values.latitude;
				markerData["lon"] = items[itm].values.longitude;
				markerData["title"] = currTitle;
				markerData["html"] = currHTML;
				arrayAppend(markerArr,markerData);
				if ( LEN(TRIM(animatedState)) )
					markerData["animation"] = animatedState;
			}

			// Convert the Marker Data to JSON array object
			markerSet = serializeJSON(markerArr);

		</cfscript>

		<cfreturn markerSet>

	</cffunction>
	
</cfcomponent>